package com.docus.server.mzsy.job;

import com.alibaba.fastjson.JSONObject;
import com.docus.core.util.Func;
import com.docus.infrastructure.core.utils.TableJsonRead;
import com.docus.infrastructure.web.api.CommonResult;
import com.docus.infrastructure.web.api.ResultCode;
import com.docus.server.mzsy.entity.MzsyReportDataView;
import com.docus.server.mzsy.mapper.MzsyReportDataViewMapper;
import com.docus.server.rpc.DownPlatformService;
import com.docus.server.rpc.TaskDistributeService;
import com.docus.server.rpc.dto.*;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

/**
 * @author YongBin Wen
 * @date 2024/8/16 14:45
 */
@Component
@Slf4j
public class MzsyReportCollectJob {

    @Autowired
    private DownPlatformService downPlatformService;
    @Autowired
    private TaskDistributeService taskDistributeService;
    @Autowired
    private MzsyReportDataViewMapper reportDataViewMapper;

    @XxlJob("MzsyReportCollectJob")
    public void reportCollect() {
        // 1.获取任务和患者信息 2.查询视图信息 3.上报下载
        String jobParam = XxlJobHelper.getJobParam();
        JSONObject jobParamJsonObject = JSONObject.parseObject(jobParam);
        String collectorId = jobParamJsonObject.getString("collectorId");
        String extraCondition = jobParamJsonObject.getString("extraCondition");
        if (Func.isBlank(collectorId)) {
            log.warn(">>>>>>>>>>>>>>>>>>>>>> 梅州三院 检查、检验、心电等报告视图采集,未配置采集器获取任务信息");
            return;
        }

        CommonResult<ReportDownTwoDto> commonResult = taskDistributeService.getNoViewTaskByCollectorId(collectorId);
        if (ResultCode.SUCCESS.getCode().equals(commonResult.getCode())
                && commonResult.getData() != null) {
            ReportDownTwoDto downTwoDto = commonResult.getData();
            ReportTaskTwoDto task = downTwoDto.getTasks().get(0);
            log.info(">>>>>>>>>>>>>>>>>>>>>> 梅州三院 检查、检验、心电等报告视图采集任务，获取了任务：{}", Func.toJson(task));

            ReportHospitalTwoDto hospitalTwoDto = downTwoDto.getHospitals().get(0);
            ReportPatientTwoDto reportPatientTwoDto = downTwoDto.getPatient();
            String jzh = downTwoDto.getJzh();

            List<MzsyReportDataView> mzsyReportDataViewList = reportDataViewMapper.findByPatJzhAndOtherCondition(jzh, extraCondition);
            int size = mzsyReportDataViewList.size();
            log.info("住院号：{}，住院次数：{},记账号：{} 采集视图数据：{} 条！数据：{}", reportPatientTwoDto.getInpatientNo(), hospitalTwoDto.getAdmissTimes(), jzh, size,Func.toJson(mzsyReportDataViewList));
            if (size <= 0) {
                return;
            }
            TableJsonRead tableJsonRead = new TableJsonRead();
            JSONObject moduleAlisConfig = tableJsonRead.Read("data-config", "mzsy-module-alis", JSONObject.class);

            ReportDownPatientDto patient = new ReportDownPatientDto();
            patient.setPatientid(downTwoDto.getPatientId());

            ReportDownDto reportDownDto = new ReportDownDto();
            reportDownDto.setCollectorid(collectorId);
            reportDownDto.setIp("");
            reportDownDto.setPatient(patient);
            int sort=0;
            for (MzsyReportDataView mzsyReportDataView : mzsyReportDataViewList) {
                ReportDownScanFileDto reportDownScanFileDto = new ReportDownScanFileDto();
                reportDownScanFileDto.setDownurl(mzsyReportDataView.getFfilePath());
                reportDownScanFileDto.setFiletitle(mzsyReportDataView.getFfileName());
                reportDownScanFileDto.setSerialnum(mzsyReportDataView.getFfileId());
                reportDownScanFileDto.setFilesource(1);
                reportDownScanFileDto.setFiletype(1);
                reportDownScanFileDto.setFilestoragetype(1);
                reportDownScanFileDto.setTaskid(task.getTaskId());
                reportDownScanFileDto.setSort(++sort);
                List<ReportDownScanFileDto> scanFiles = Collections.singletonList(reportDownScanFileDto);

                String assortId;
                String fpatModule = mzsyReportDataView.getFpatModule();
                if (Func.isBlank(fpatModule)) {
                    assortId = moduleAlisConfig.getString("default");
                } else {
                    assortId = moduleAlisConfig.getString(fpatModule.toLowerCase().trim());
                    if (Func.isBlank(assortId)) {
                        assortId = moduleAlisConfig.getString("default");
                    }
                }
                reportDownDto.setAssortid(assortId);
                reportDownDto.setScanfiles(scanFiles);
                downPlatformService.report(reportDownDto);
            }
        }
    }

    public static void main(String[] args) {
        TableJsonRead tableJsonRead = new TableJsonRead();
        JSONObject moduleAlisConfig = tableJsonRead.Read("data-config", "mzsy-module-alis", JSONObject.class);
        System.out.println(moduleAlisConfig);
    }
}
